@@ -38,7 +38,7 @@
                 instF <- model.frame(formula, h, na.action="na.pass",
                                      drop.unused.levels=TRUE)
         }
-    h <- lapply(1:ncol(Y), function(i) formula(attr(instF, "terms"), .GlobalEnv))
+    h <- lapply(1:ncol(Y), function(i) formula(attr(instF, "terms")))
     data <- cbind(modelF, instF)
     data <- data[,!duplicated(colnames(data))]
     return(.slModelData(g,h,data,survOptions, vcovOptions,na.action))
@@ -49,8 +49,9 @@
     {
         modelF <- model.frame(formula, data, na.action="na.pass",
                               drop.unused.levels=TRUE)
+        
         if (is.matrix(modelF[[1]]))
-            return(.multiToSys(formula, h, data))
+            return(.multiToSys(formula, h, data, survOptions, vcovOptions, na.action))
         parNames <- colnames(model.matrix(attr(modelF, "terms"), modelF))
         k <- length(parNames)
         if (any(class(h) == "formula"))
